##############################################################################
#
# Copyright (c) 2001 Alexandre Fernandez (alex@grotte.org). All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
#############################################################################
from elementtree.ElementTree import tostring 
from elementtree.ElementTree import fromstring
from elementtree.ElementTree import Element

"""Avatar definition and managment

"""
class Avatar:
    """An avatar is a representation in memory of somebody
    on the chat.
    Only a paramater for the moment : the nickname
    """
    nickname = None
    socket = None
    myRoom = None
    address = None
    
    def __init__(self, socket, nickname, address=None, myRoom=None):
        self.socket = socket
        self.nickname = nickname
        self.myRoom = None
        self.address = address

    def getSocket(self):
        """
        @return: the socket of the avatar
        @rtype: L{socket}
        """
        return self.socket

    def join(self, room):
        """Make join the room

        @return: None
        
        @param room: room to join
        @type room: L{Room}
        
        """
        self.myRoom = room
        self.myRoom.join(self)

    def part(self):
        """ Make part the curent room
        @return: None
        """
        self.myRoom.part(self)
        self.myRoom = None

    def getRoom(self):
        """
        @return: The default room
        @rtype: L{Room}
        """
        return self.myRoom

    def getNickname(self):
        """
        @return: The nickname of the avatar
        @rtype: C{string}
        """
        return self.nickname

    def setNickname(self, nickname):
        """Define the nickname of the avatar
        @param nickname: the new nickname
        @type nickname: C{string}
        """
        old_nickname = self.getNickname()
        self.nickname = nickname 
	myRoom = self.getRoom()
	myRoom.send('<nickname from="%s">%s</nickname>' % (old_nickname, 
	                                                   nickname))

    def sendNickName(self):
        """
        Send its nickname definition to the avatar
        """
        nickname = Element('nickname')
        nickname.text = self.getNickname()
        nickname_xml = tostring(nickname)
        self.send(nickname_xml)
        
    def send(self, message):
        """Send a message to the avatar client side
        @param message: the message to send
        @type message: C{string}
        """
        if message[-1] != '\0':
            message = message + '\0'
        self.socket.send(message)


    def sendToRoom(self, message):
        """Send a message to the avatar's room
        @param message: The message to send
        @type: message: C{string}
        """
        myRoom = self.getRoom()
        myRoom.send(message)


    def getInfoXML(self):
        """
        @return: the XML representation of the avatar
        @rtype: C{string}
        """
        info = Element('avatar')
        nickname = Element('nickname')
        nickname.text = self.nickname
        info.append(nickname)
        
        return tostring(info)

    def __str__(self):
        return self.getInfoXML()
