##############################################################################
#
# Copyright (c) 2001 Alexandre Fernandez (alex@grotte.org). All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
#############################################################################
from Globals import HTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass
from Globals import DTMLFile
from OFS.SimpleItem import SimpleItem
from HappyServer import HappyServer
from urllib import urlencode
from AccessControl import getSecurityManager

class HappyChatClientObject(SimpleItem):
    """
    Object to use in Zope to have a HappyChat Client displayed
    """
    
    meta_type = "HappyChat Client object"

    manage_options = (
        ({'label' : 'Chat', 'action' : 'manage_chat'},) + 
        SimpleItem.manage_options
        )

    manage_chat = DTMLFile('zope/zmi/HappyChatClientObject_manage_chat', globals())
    chat_view_template = PageTemplateFile('pt/HappyChatClientObject_view.pt', globals())
    
    def __init__(self, id, title='', hostname='localhost', port=1234):
        self.id = id
        self.title = title
        self.hostname = hostname
        self.port = port

    def index_html(self):
        """return the content of html view to show swf file
        """
        user_id = 'no_nickname'
        return self.chat_view(nickname=user_id)
    def getUserId(self):
        """
        Get the user id or nickname
        @return: User Id / Nickname of client
        @rtype: C{string}
        """
        return str(getSecurityManager().getUser())
         
    def getChatInfo(self):
        """
        get chat information
        """
        server = HappyServer()
        return server.getInfo()
    
    def chat_view(self, REQUEST=None):
        """
        """
        server = HappyServer()
        hostname, port = server.getConfig()
        
        user_id = self.getUserId()
        session_id = server.createSession(nickname=user_id)
        #session_id = 'toto'
        flashvars = { 'session_id' : session_id,
                      'hostname' : hostname  ,
                      'port' : port
                    }
        
        flashvars = urlencode(flashvars)
        return self.chat_view_template(REQUEST, flashvars=flashvars)
        
    
InitializeClass(HappyChatClientObject)

addHappyChatClientObjectForm = DTMLFile('zope/zmi/addHappyChatClientObjectForm',
                                     globals())

def addHappyChatClientObject(container, id, title, hostname='localhost',
                          port=1234, REQUEST=None):
    """
    HappyChat Client Object factory
    Store a new object in container
    
    @param id: Identifier in the container of the new object
    @type id: C{string}

    @param title: Title of the new object
    @type title: C{string}

    @param hostname: Hostname of the chat server, 'localhost' by default
    @type hostname: C{string}

    @param port: port of the chat server, 1234 by default
    @type port: C{int}
    """
    id = str(id)
    title = str(title)
    hosname = str(hostname)
    port = int(port)
    
    chatObject = HappyChatClientObject(id, title, hostname, port)
    container._setObject(id, chatObject)
    obj = container._getOb(id)
    
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(obj.absolute_url() + '/manage_workspace')
