##############################################################################
#
# Copyright (c) 2004 Alexandre Fernandez (alex@grotte.org). All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
import random
import base64
from zLOG import LOG, DEBUG

def HappyChatZopeSessionManager():
    """
    Session manager singleton,
    this tool deal with client and server to store data from Zope
    session are unique, and store temporarly in this tool before avatar
    creation.
    """
    if _HappyChatZopeSessionManager.instance == None:
        return _HappyChatZopeSessionManager()
    else:
        return _HappyChatZopeSessionManager.instance

class _HappyChatZopeSessionManager:

    instance = None

    def __init__(self):
        self.sessions = {}
        _HappyChatZopeSessionManager.instance = self

    def _generateId(self):
        random_str = str(random.random()).split('.')[1]
        session_id = base64.encodestring(random_str).strip()
        return session_id.replace('=','')
    
    def _createUniqueSession(self):
        """Create an unique id and reserve it
        @return: the session object
        @rtype: L{HappyChatZopeSession}
        
        """
        session_id = self._generateId()
        LOG("HappyChat" , DEBUG, 'new session_id = ' + session_id)
        while session_id in self.sessions.keys():
            session_id = self._generateId()
            LOG("HappyChat" , DEBUG, 'new session_id = ' + session_id)
            
        session = HappyChatZopeSession(session_id)
        self.sessions[session_id] = session

        return session
        
    def createSession(self, nickname):
        """
        Create a session structure and return an unique session id
        @return: unique session id
        @rtype: C{string}
        """
        session = self._createUniqueSession()
        session.setNickName(nickname)
        session_id = session.getSessionId()
        
        return session_id
    def getSession(self, session_id):
        """
        Get the Session
        @param session_id: Session string identifier
        @type session: C{string}
        """
        return self.sessions.get(session_id)
    
    def delSession(self, session_id):
        """
        Remove a session by its session identifier
        @param session_id: Session identifier to destroy
        @type session_id: C{string}
        """
        if self.sessions.has_key(session_id):
            del self.sessions[session_id]





class HappyChatZopeSession:

    """
    Sessions are here temporarly data structure where store avatar informations
    before avatar creation.
    The HappyChat Zope Session is create when people ask to view client applet.
    session id is passed to the applet, and next, applet give session id to get
    avatar information (here, just nickname)
    """
    
    def __init__(self, session_id, nickname=None, address=None):
        self.session_id = session_id
        self.nickname = nickname
        self.address = address

    def setNickName(self, nickname):
        """
        Set session nickname
        @param nickname: New Nickname of the session
        @type nickname: C{string}
        @return: None
        
        """
        self.nickname = nickname
        
    def setAddress(self, address):
        """
        Set the IP address of the client
        @param address: IP adress or dns host name
        @type address: C{string}
        @return: None
        """
        self.address = address
        
    def getSessionId(self):
        """
        Get the unique identifier of the session
        
        @return: session string identifier
        @rtype: C{string}
        """
        return self.session_id

    def getNickName(self):
        return self.nickname

    def getAddress(self):
        return self.address
    
    def __str__(self):
        """
        Build an XML structure for this session
        @return: the XML representation of the session
        @rtype: C{string}
        """
        xml = ''
        return xml

        
